import React from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {useDispatch} from 'react-redux';
import classNames from 'classnames';
import ApiButton from '../api-button';
import {localeMessages} from '../../../services/locale-service';
import DisplayComponent from '../../sink/display-component';
import ToggleComponent from '../toggle-component';
import {XlReactWidgetDate} from '../../../xl-react-widget/xl-react-widget-date/xl-react-widget-date';
import RU from '../../../utils/ramda-utils';
import DisplaySelectComponent from './display-select-component';
import DisplayEnumComponent from './display-enum-component';

export const bindingTypes = {
    ACTION: 'action',
    API: 'api',
    BOOLEAN: 'boolean',
    DATE: 'date',
    DISPLAY: 'display',
    ENUM: 'enum',
    INPUT: 'input',
    INT: 'integer',
    SELECT: 'select'
};

export const ComponentBindings = (props) => {
    const dispatch = useDispatch();
    const {
        api,
        bindingItems,
        componentProps,
        modelName,
        resetPristinity,
        updateProps
    } = props;

    const createBindingSection = (sectionConfig, index) => {
        const updateComponentProps = (value) =>
            updateProps(
                {...R.assocPath(sectionConfig.name.split('.'), value, {})},
                {
                    name: sectionConfig.name,
                    value
                }
            );

        const onIntegerInputChange = (event) => {
            updateProps(parseInt(event.target.value, 10));
        };

        const toggleBooleanValue = (event) => {
            updateComponentProps(event.target.checked);
        };

        const onInputChange = (event) => {
            updateComponentProps(event.target.value);
        };

        const handleModelChange = (value) => {
            RU.invoke(sectionConfig.onChangeCallApi, [value], api);
            updateComponentProps(value);
        };

        const onResetPristinity = () => {
            setTimeout(() => {
                if (resetPristinity) {
                    resetPristinity();
                } else {
                    RU.invoke('resetPristinity', [], api);
                }
            });
        };

        const onFocus = () => {
            RU.invoke('focusOnFirstInvalidField', [], api);
        };

        return R.cond([
            [R.whereEq({type: bindingTypes.DISPLAY}), ({className, label, name}) => {
                const value = R.prop(name.split('.'), componentProps);

                return <DisplayComponent className={className} key={index} label={label} value={value}/>;
            }],
            [R.whereEq({type: bindingTypes.INT}),
                R.always(
                    <div key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <input
                            onChange={onIntegerInputChange} type="number"
                            value={sectionConfig.value || componentProps.model}
                        />
                    </div>
                )],
            [R.whereEq({type: bindingTypes.BOOLEAN}),
                (config) => <ToggleComponent
                    checked={config.value}
                    clazzName={config.className}
                    handleToggle={(event) => {
                        if (config.action) {
                            dispatch({
                                ...config.action,
                                modelName
                            });
                        }
                        toggleBooleanValue(event);
                    }}
                    key={index}
                    label={config.label}
                />
            ],
            [R.whereEq({type: bindingTypes.DATE}),
                () =>
                    <div className={sectionConfig.className} key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <XlReactWidgetDate model={sectionConfig.value} onModelChange={handleModelChange}/>
                    </div>
            ],
            [R.whereEq({type: bindingTypes.ENUM}),
                (config) => <DisplayEnumComponent
                    clazzName={config.className}
                    currentValue={config.value}
                    key={index}
                    label={config.label}
                    listOfValues={config.enum}
                    name={config.name}
                    onInputChange={onInputChange}
                />],
            [R.whereEq({type: bindingTypes.SELECT}),
                (config) => <DisplaySelectComponent
                    clazzName={config.className}
                    currentValue={config.value}
                    key={index}
                    label={config.label}
                    listOfValues={config.values}
                    name={config.name}
                    onSelectChange={handleModelChange}
                />],
            [R.whereEq({type: bindingTypes.INPUT}),
                R.always(
                    <div
                        className={
                            classNames(R.replace('.', '-', R.or(sectionConfig.name, '')), sectionConfig.className)
                        }
                        key={index}>
                        <span className="label">{sectionConfig.label}</span>
                        <input
                            onChange={onInputChange}
                            type="text"
                            value={sectionConfig.value}
                        />
                        <ApiButton
                            clazzName={'apply-value-btn'}
                            handleClick={handleModelChange.bind(this, sectionConfig.value)}
                            label={sectionConfig.buttonLabel}
                        />
                    </div>
                )],
            [R.whereEq({type: bindingTypes.API}),
                () =>
                    <div key={index}>
                        <span className="label">{localeMessages.apiLabel}</span>
                        <ApiButton
                            clazzName={'reset-pristinity-btn'}
                            handleClick={onResetPristinity}
                            label={localeMessages.resetPristinityButton}
                        />
                        <ApiButton
                            clazzName={'focus-btn'}
                            handleClick={onFocus}
                            label={localeMessages.focusLabel}
                        />
                    </div>
            ],
            [R.whereEq({type: bindingTypes.ACTION}),
                () =>
                    <div key={index}>
                        <span className="label">{localeMessages.actionLabel}</span>
                        {bindingItems.map((item) => <ApiButton
                            clazzName={item.className}
                            handleClick={() => dispatch(item.value)}
                            key={item.label}
                            label={item.label}
                        />)}
                    </div>
            ]
        ])(sectionConfig);
    };

    return <div className="testBindings">
        {bindingItems ? RU.mapIndexed(createBindingSection, bindingItems) : null}
    </div>;
};

ComponentBindings.propTypes = {
    api: PropTypes.object,
    bindingItems: PropTypes.arrayOf(PropTypes.shape({
        className: PropTypes.string,
        label: PropTypes.string.isRequired,
        name: PropTypes.string.isRequired,
        type: PropTypes.oneOf(R.values(bindingTypes)).isRequired,
        value: PropTypes.any
    })),
    componentProps: PropTypes.object.isRequired,
    modelName: PropTypes.string.isRequired,
    resetPristinity: PropTypes.func,
    updateProps: PropTypes.func.isRequired
};
