import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import DisplayComponent from '../sink/display-component';
import {localeMessages} from '../../services/locale-service';
import {generateLazyChildren} from './playground-utils';
import ApiButton from './api-button';
import ElementNavTitle from './element-nav-title';

import '../../images/udm.Application.svg';

export default class TreeTestComponent extends PureComponent {
    static propTypes = {
        api: PropTypes.object,
        children: PropTypes.oneOfType([
            PropTypes.element,
            PropTypes.array
        ]),
        className: PropTypes.string,
        currentState: PropTypes.object.isRequired,
        modelName: PropTypes.string.isRequired,
        showBindings: PropTypes.bool,
        updateState: PropTypes.func.isRequired
    };

    onCollapseAll = () => {
        const rootChildren = this.tree.getChildNodes(this.tree.getRootNode());
        R.forEach((node) => {
            this.tree.closeNode(node);
        }, rootChildren);
    };

    onNodesSelected = (selectedNodes) => {
        const {updateState, modelName, currentState} = this.props;
        updateState(modelName, {...currentState[modelName], selectedNodes});
    };

    loadNodes = (parentNode, done) => setTimeout(() => {
        done(null, generateLazyChildren(parentNode));
    }, 10000);

    stripModelForPrint = (tree) => {
        const modifiedTree = R.clone(tree);

        const modify = (data) => {
            const children = Array.isArray(data) ? data : data.children;
            const modifyParentNode = (node) => {
                if (node.parent) {
                    node.parentId = node.parent.id;
                }
            };
            const removeEmptyChildren = (node) => {
                if (R.isEmpty(node.children)) {
                    node.children = undefined;
                }
            };
            const removeRedundantKey = (node, key) => {
                if (!R.includes(key, ['children', 'id', 'name', 'parentId'])) {
                    node[key] = undefined;
                }
            };
            R.forEach((node) => {
                const keys = R.keys(node);
                R.forEach((key) => {
                    modifyParentNode(node);
                    removeEmptyChildren(node);
                    removeRedundantKey(node, key);
                }, keys);

                if (node.children) {
                    modify(node);
                }
            }, children);
        };

        modify(modifiedTree);

        return modifiedTree;
    };

    ref = (el) => {
        this.tree = el.tree;
    };

    render() {
        const {children, className, currentState, modelName, showBindings} = this.props;
        const componentState = currentState[modelName];
        const {description, title, type} = componentState.navigation;

        const newProps = {
            loadNodes: this.loadNodes,
            model: componentState.model,
            onNodesSelected: this.onNodesSelected,
            selectedNodes: componentState.selectedNodes,
            selectionMode: componentState.selectionMode,
            treeRef: this.ref
        };

        const clonedChildren = React.Children.map(children, (child) => React.cloneElement(child, newProps));
        const displayModel = this.stripModelForPrint(componentState.model);

        const displayBindings = () => showBindings ? <div className="testBindings">
            <div>
                <span className="label">{localeMessages.apiLabel}</span>
                <ApiButton
                    clazzName={'collapse-all-btn'}
                    handleClick={this.onCollapseAll}
                    label={localeMessages.collapseAll}
                />
            </div>
            <DisplayComponent
                className="model"
                label={localeMessages.modelLabel}
                value={displayModel}
            />
        </div> : null;

        return <div>
            <ElementNavTitle
                description={description}
                modelName={modelName}
                title={title}
                type={type}
            />
            <div className={className}>
                <div className="testComponent">
                    {clonedChildren}
                </div>
                {displayBindings()}
            </div>
        </div>;
    }
}
