import React, {useCallback, useState} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import debounce from 'debounce';
import {localeMessages} from '../../services/locale-service';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {validationMessageClasses} from '../../utils/css-utils';
import {XlReactWidgetAlertMessage} from
    '../../xl-react-widget/xl-react-widget-alert-message/xl-react-widget-alert-message';
import {XlReactFormItemControl} from '../../xl-react-form/xl-react-form-item-control';
import {ScrollableContainer} from './scrollable-container';
import {TopPanelComponent} from './top-panel';

import './xl-react-dip-set-of-strings.less';

export const INPUT_DEBOUNCE_TIME = 300;

export function XlReactDipSetOfStringsComponent(props) {
    const {
        autoHide,
        assignTableRef,
        assignTableWrapperRef,
        assignRef,
        description,
        disabled,
        duplicateKey,
        emptyKey,
        encrypted,
        canAddNewItem,
        cellState,
        handleAddRow,
        handleCloseButton,
        handleNewKeyBlur,
        handleNewKeyChange,
        handleNewKeyFocus,
        handleSortChange,
        label,
        message,
        model,
        modelKeys,
        newKey,
        onCleaningSearchWord,
        onKeyBlur,
        onRemoveKey,
        onSearchChanged,
        pristine,
        required,
        searchWord,
        showNewKeyValueRow,
        updateCellState,
        valid,
        sorting
    } = props;

    const [typedKey, setTypedKey] = useState(searchWord);

    useCallback(() => {
        setTypedKey(searchWord);
    }, [searchWord]);

    const debounceSearchChanged = debounce(onSearchChanged, INPUT_DEBOUNCE_TIME);

    const onSearchChange = (event) => {
        const newValue = event.target.value;
        setTypedKey(newValue);
        debounceSearchChanged(newValue);
    };

    const displayKeyError = <XlReactWidgetAlertMessage
        hide={R.isNil(duplicateKey) && R.isNil(emptyKey)}
        internalState="error">
        {duplicateKey ? localeMessages.keyAlreadyInUse(duplicateKey) : localeMessages.emptyKeyErrorMessage}
    </XlReactWidgetAlertMessage>;

    const topPanel = () => <TopPanelComponent
        canAddNewItem={canAddNewItem}
        disabled={disabled}
        handleAddRow={handleAddRow}
        handleCleaningSearchWord={onCleaningSearchWord}
        handleSearchChanged={onSearchChange}
        searchWord={typedKey}
    />;

    const hideContainer = () => autoHide !== false && R.isEmpty(modelKeys) && !showNewKeyValueRow;
    const container = () => !hideContainer() && <ScrollableContainer
        assignRef={assignRef}
        assignTableRef={assignTableRef}
        assignTableWrapperRef={assignTableWrapperRef}
        cellState={cellState}
        disabled={disabled}
        encrypted={encrypted}
        handleCloseButton={handleCloseButton}
        handleKeyBlur={onKeyBlur}
        handleNewKeyBlur={handleNewKeyBlur}
        handleNewKeyChange={handleNewKeyChange}
        handleNewKeyFocus={handleNewKeyFocus}
        handleRemoveKey={onRemoveKey}
        handleSortChange={handleSortChange}
        hasKeyError={Boolean(duplicateKey || emptyKey)}
        message={message}
        model={model}
        modelKeys={modelKeys}
        newKey={newKey}
        pristine={pristine}
        searchWord={searchWord}
        showNewKeyValueRow={showNewKeyValueRow}
        sorting={sorting}
        updateCellState={updateCellState}
        valid={valid}
    />;

    const controlProps = {
        controlClass: 'xl-react-dip-set-of-strings',
        customControlClasses: validationMessageClasses(valid, message),
        description,
        disabled,
        label,
        message,
        required
    };

    return <XlReactFormItemControl {...controlProps}>
        {topPanel()}
        {displayKeyError}
        {container()}
    </XlReactFormItemControl>;
}

XlReactDipSetOfStringsComponent.propTypes = {
    assignRef: PropTypes.func,
    assignTableRef: PropTypes.func,
    assignTableWrapperRef: PropTypes.func,
    autoHide: PropTypes.bool,
    canAddNewItem: PropTypes.bool,
    cellState: PropTypes.shape({
        column: PropTypes.oneOf(['key']),
        row: PropTypes.number,
        value: PropTypes.oneOfType([PropTypes.string, PropTypes.number])
    }),
    description: PropTypes.string,
    disabled: PropTypes.bool,
    duplicateKey: PropTypes.string,
    emptyKey: PropTypes.bool,
    encrypted: PropTypes.bool,
    handleAddRow: PropTypes.func,
    handleCloseButton: PropTypes.func,
    handleNewKeyBlur: PropTypes.func,
    handleNewKeyChange: PropTypes.func,
    handleNewKeyFocus: PropTypes.func,
    handleNewValueBlur: PropTypes.func,
    handleSortChange: PropTypes.func,
    label: PropTypes.string,
    message: xlMessageTypeProp,
    model: PropTypes.array,
    modelKeys: PropTypes.array,
    newKey: PropTypes.string,
    onCleaningSearchWord: PropTypes.func,
    onKeyBlur: PropTypes.func,
    onRemoveKey: PropTypes.func,
    onSearchChanged: PropTypes.func,
    onValidityChange: PropTypes.func,
    pristine: PropTypes.bool,
    required: PropTypes.bool,
    searchWord: PropTypes.string,
    showNewKeyValueRow: PropTypes.bool,
    sorting: PropTypes.object,
    updateCellState: PropTypes.func,
    valid: PropTypes.bool
};

XlReactDipSetOfStringsComponent.defaultProps = {canAddNewItem: true, encrypted: false};
