import PropTypes from 'prop-types';
import R from 'ramda';
import React, {Component} from 'react';

import classNames from 'classnames';
import {
    Col, MenuItem, Nav, DropdownButton, NavItem, Row, Tab
} from 'react-bootstrap';
import {XlReactWidgetSpinner} from '../xl-react-widget/xl-react-widget-spinner/xl-react-widget-spinner';
import RU from '../utils/ramda-utils';
import {nextId} from '../utils/id-generator';
import {
    handleTabResize,
    handleTabUnmount,
    repositionElementsInTabNavigation
} from './xl-react-tab-system-dom-manipulation';

import './skins/all-tab-skins.less';

const TABS_COLUMN_SIZE = 12;

export const skins = {
    BLUE: 'blue',
    DEFAULT: 'default',
    NONE: 'none',
    UNDERLINED: 'underlined'
};

export const isCloseableTabSkin = (skin) => R.includes(skin, [skins.DEFAULT, skins.NONE]);
export const canDropDownBeShown = (skin) => R.includes(skin, [skins.DEFAULT, skins.NONE]);
export const canNewTabBtnBeShown = (skin) => R.includes(skin, [skins.DEFAULT, skins.NONE]);
export const canBeRepositioned = (skin) => R.includes(skin, [skins.DEFAULT, skins.NONE]);

export class XlReactTabSystem extends Component {
    static propTypes = {
        actions: PropTypes.arrayOf(PropTypes.shape({
            action: PropTypes.func.isRequired,
            label: PropTypes.string.isRequired
        })),
        activeTabId: PropTypes.string,
        data: PropTypes.arrayOf(PropTypes.shape({
            fullTitle: PropTypes.string,
            iconUrl: PropTypes.string,
            loading: PropTypes.bool,
            pristine: PropTypes.bool.isRequired,
            tabId: PropTypes.string.isRequired,
            title: PropTypes.string.isRequired,
            viewType: PropTypes.string.isRequired
        })).isRequired,
        loaderElement: PropTypes.element,
        onCloseTab: PropTypes.func,
        onNewTab: PropTypes.func,
        onTabSelected: PropTypes.func,
        skin: PropTypes.string.isRequired,
        tabDefinitions: PropTypes.object.isRequired
    };

    static defaultProps = {actions: [], config: {}};

    constructor(props) {
        super(props);
        this.state = {menuVisible: false, tabSystemId: `id-${nextId()}`};
    }

    componentDidMount() {
        this.handleTabPresentation();
        if (this.props.skin === skins.DEFAULT) {
            handleTabResize(this.state.tabSystemId);
        }
    }

    componentDidUpdate() {
        this.handleTabPresentation();
    }

    componentWillUnmount() {
        handleTabUnmount();
    }

    handleActionSelected = (action) => {
        this.setState({menuVisible: false});
        action();
    };

    handleDropDownToggle = (isOpen) => {
        this.setState({menuVisible: isOpen});
    };

    handleTabPresentation = () => {
        if (canBeRepositioned(this.props.skin)) {
            repositionElementsInTabNavigation(this.state.tabSystemId);
        }
    };

    handleTabSelected = (tabId) => {
        const {activeTabId, data} = this.props;
        if (tabId !== activeTabId) {
            RU.invoke('onTabSelected', [R.find(R.propEq('tabId', tabId), data)], this.props);
            if (this.state.menuVisible) {
                this.setState({menuVisible: false});
            }
        }
    };

    generateChildId = (eventKey, type) => `${type}-${eventKey}`;

    render() {
        const {
            actions, activeTabId, data, loaderElement, onNewTab, skin, tabDefinitions
        } = this.props;
        const buildCloseTabButton = ({tabId}) => {
            const handleClose = (event) => {
                event.preventDefault();
                event.stopPropagation();
                RU.invoke('onCloseTab', [tabId, tabDefinitions], this.props);
            };
            return isCloseableTabSkin(skin) ? <i className="xl-icon close-icon" onClick={handleClose}/> : null;
        };

        const showNotPristinity = (info) => skin === skins.DEFAULT && !info.pristine ?
            <span className="asterisk">*</span> : null;

        const tabIcon = (iconUrl) => iconUrl ?
            <i className="tab-icon" style={{backgroundImage: `url(${iconUrl})`}}/> : null;

        const tabsContentStyle = (tabId) => {
            const isActiveTab = tabId === activeTabId;
            return isActiveTab ? {} : {display: 'none'};
        };

        const divider = () => !R.isEmpty(actions) && !R.isEmpty(data) ?
            <MenuItem divider={true}/> : null;

        const getHoverTitle = (info) => info.fullTitle || info.title;

        const dropDownWithListOfTabs = () => {
            if (canDropDownBeShown(skin)) {
                return <DropdownButton
                    id={`drop-down-${this.state.tabSystemId}`}
                    onToggle={this.handleDropDownToggle}
                    open={this.state.menuVisible}
                    title="">
                    <div className="actions">
                        {R.map((action) => <MenuItem
                            eventKey={action.action}
                            key={action.label}
                            onSelect={this.handleActionSelected}>
                            {action.label}
                        </MenuItem>, actions)}
                        {divider()}
                    </div>
                    <div className="tabs" role="menu">
                        {R.map((info) => <MenuItem
                            className={classNames({active: R.equals(info.tabId, activeTabId)})}
                            eventKey={info.tabId}
                            key={info.tabId}
                            onSelect={this.handleTabSelected}>
                            <span title={getHoverTitle(info)}>{info.title}</span>
                        </MenuItem>, data)}
                    </div>
                </DropdownButton>;
            }
            return null;
        };

        const tabTitleClass = (info) => classNames('tab-title', {'has-icon': info.iconUrl});

        const tabHeaders = R.map((info) =>
            <NavItem
                eventKey={info.tabId}
                key={info.tabId}
                role="tab">
                {tabIcon(info.iconUrl)}
                <span
                    className={tabTitleClass(info)}
                    title={getHoverTitle(info)}>
                    {info.title}
                </span>
                {buildCloseTabButton(info)}
                {showNotPristinity(info)}
            </NavItem>
        );

        const addNewTabButton = () => onNewTab && canNewTabBtnBeShown(skin) ?
            <button className="btn-add-new-tab" onClick={onNewTab}><i className="xl-icon add-icon icon-l"/></button> :
            null;

        const renderLoader = () => loaderElement ? loaderElement : <XlReactWidgetSpinner show={true}/>;

        const tabContent = R.map((info) =>
            <Tab.Pane
                eventKey={info.tabId}
                key={info.tabId}
                style={tabsContentStyle(info.tabId)}
                unmountOnExit={false}>
                {tabDefinitions[info.viewType].renderContent(info)}
                {info.loading && renderLoader()}
            </Tab.Pane>
        );

        const skinClassName = `${skin}-skin`;

        const tabSystemClassName = classNames(
            'xl-react-components',
            'xl-react-tab-system',
            this.state.tabSystemId,
            skinClassName);

        return <div className={tabSystemClassName} data-tabsystemid={this.state.tabSystemId}>
            <Tab.Container
                activeKey={activeTabId}
                generateChildId={this.generateChildId}
                onSelect={this.handleTabSelected}>
                <Row className="clearfix">
                    <Col className="header-column" sm={TABS_COLUMN_SIZE}>
                        <Nav bsStyle="tabs">
                            {tabHeaders(data)}
                        </Nav>
                        <div className="btn-group-backdrop">
                            {addNewTabButton()}
                            {dropDownWithListOfTabs()}
                        </div>
                    </Col>
                    <Col className="content-column" sm={TABS_COLUMN_SIZE}>
                        <Tab.Content animation={false}>
                            {tabContent(data)}
                        </Tab.Content>
                    </Col>
                </Row>
            </Tab.Container>
        </div>;
    }
}
