import React, {memo, useCallback, useEffect, useRef, useState} from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {useDebouncedFunction} from '../../utils/debounce-hook';
import {XlReactWidgetSpinner} from '../xl-react-widget-spinner/xl-react-widget-spinner';

import './xl-react-widget-lazy-loaded-tree.less';
import {XlReactWidgetTree} from '../xl-react-widget-tree/xl-react-widget-tree';
import {getNodeFromEvent} from '../xl-react-widget-tree/xl-react-widget-tree-service';

const XlReactWidgetLazyLoadedTreeComponent = (props) => {
    const {
        className,
        hasMore,
        isLoadingNewNodes,
        newNodes,
        onScrollEnd,
        scrollEndThreshold
    } = props;

    const tree = useRef();
    const [isDebounce, setIsDebounce] = useState(false);

    const handleOnScrollEndCall = () => {
        onScrollEnd();
        setIsDebounce(false);
    };

    const handleScrollEnd = useDebouncedFunction(handleOnScrollEndCall, 300);

    const handleScroll = useCallback(() => {
        const shouldNotHandleScroll = () => !hasMore || isLoadingNewNodes;
        if (shouldNotHandleScroll()) {
            return;
        }
        const $scrollElement = $(tree.current.scrollElement);
        const scroll = $scrollElement.height() + $scrollElement.scrollTop() + scrollEndThreshold;
        const progress = scroll / $scrollElement.find('.infinite-tree-content').height() * 100 || 0;
        if (progress >= 100) {
            setIsDebounce(true);
            handleScrollEnd();
        }
    }, [hasMore, isLoadingNewNodes]);

    useEffect(() => {
        const $scrollElement = $(tree.current.scrollElement);
        if (onScrollEnd) {
            $scrollElement.on('scroll', handleScroll);
        }
        return () => $scrollElement.off('scroll');
    }, [handleScroll]);

    useEffect(() => {
        RU.invoke('registerNodeListeners', [tree.current], props);
        tree.current.on('contentDidUpdate', () => {
            RU.invoke('registerNodeListeners', [tree.current], props);
        });
    }, []);

    useEffect(() => {
        tree.current.addChildNodes(newNodes);
    }, [newNodes]);

    const invokeClickHandler = (handlerName, node) => {
        if (RU.isDefined(node.id)) {
            RU.invoke(handlerName, [node, {dispatch: true}], props);
        }
    };

    const handleDoubleClick = (event) => {
        if (!R.includes(tree.current.options.togglerClass, event.target.className)) {
            invokeClickHandler('onDoubleClick', getNodeFromEvent(event));
        }
    };

    const handleClick = (event) => {
        if (!R.includes(tree.current.options.togglerClass, event.target.className)) {
            invokeClickHandler('onClick', getNodeFromEvent(event));
        }
    };

    const assignTree = (component) => {
        tree.current = component.tree;
        RU.invoke('treeRef', [tree.current], props);
    };

    const treeClassName = classNames(className, 'tree-container');

    return (
        <div className="xl-react-widget-lazy-loaded-tree">
            <XlReactWidgetSpinner show={isLoadingNewNodes || isDebounce}/>
            <XlReactWidgetTree
                {...props}
                className={treeClassName}
                onClick={handleClick}
                onDoubleClick={handleDoubleClick}
                treeRef={assignTree}
            />
        </div>
    );
};

XlReactWidgetLazyLoadedTreeComponent.propTypes = {
    autoOpen: PropTypes.bool,
    blocksInCluster: PropTypes.number,
    className: PropTypes.string,
    dndHandlers: PropTypes.shape({
        onDrag: PropTypes.func,
        onDragEnd: PropTypes.func,
        onDragEnter: PropTypes.func,
        onDragExit: PropTypes.func,
        onDragLeave: PropTypes.func,
        onDragOver: PropTypes.func,
        onDragStart: PropTypes.func,
        onDrop: PropTypes.func
    }),
    dontSelectClasses: PropTypes.arrayOf(PropTypes.string),
    droppableOptions: PropTypes.oneOfType([
        PropTypes.bool,
        PropTypes.shape({
            accept: PropTypes.func.isRequired,
            drop: PropTypes.func.isRequired,
            hoverClass: PropTypes.string.isRequired
        })
    ]),
    hasMore: PropTypes.bool,
    isLoadingNewNodes: PropTypes.bool,
    loadNodes: PropTypes.func,
    model: PropTypes.array,
    newNodes: PropTypes.array,
    noDataText: PropTypes.string,
    nodeHandlers: PropTypes.shape({
        onCloseNode: PropTypes.func,
        onOpenNode: PropTypes.func
    }),
    onClick: PropTypes.func,
    onDoubleClick: PropTypes.func,
    onNodesSelected: PropTypes.func,
    onScrollEnd: PropTypes.func,
    registerNodeListeners: PropTypes.func,
    rowRenderer: PropTypes.func.isRequired,
    scrollEndThreshold: PropTypes.number,
    selectable: PropTypes.bool,
    selectedNodes: PropTypes.arrayOf(PropTypes.string),
    selectionMode: PropTypes.oneOf(['single', 'multi']),
    treeRef: PropTypes.func
};

XlReactWidgetLazyLoadedTreeComponent.defaultProps = {
    blocksInCluster: 4,
    className: 'tree-container',
    scrollEndThreshold: 50
};

export const XlReactWidgetLazyLoadedTree = memo(XlReactWidgetLazyLoadedTreeComponent);
