/* eslint-disable max-lines */
import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import template from 'lodash.template';
import BootstrapTable from 'react-bootstrap-table-next';
import filterFactory, {customFilter} from 'react-bootstrap-table2-filter';
import paginationFactory from 'react-bootstrap-table2-paginator';
import {ButtonToolbar, DropdownButton, MenuItem} from 'react-bootstrap';
import classNames from 'classnames';
import {localeMessages} from '../../services/locale-service';
import RU from '../../utils/ramda-utils';
import {XlReactWidgetFilter} from '../xl-react-widget-filter/xl-react-widget-filter';
import {XlReactWidgetSpinner} from '../xl-react-widget-spinner/xl-react-widget-spinner';
import {
    newStateTransformerType,
    tableFilterType,
    tableLayoutType,
    tablePagingType,
    tableSortingType
} from './xl-react-widget-table-types';
import {extractFilters, mergeFilters, shouldShowFilters} from './xl-react-widget-table-utils';
import {setHeaderPadding} from './xl-react-widget-table-dom-manipulation';

import './xl-react-widget-table.less';
import './xl-react-widget-table-pagination.less';
import './xl-react-widget-table-dropdown.less';

export class XlReactWidgetTable extends PureComponent {
    static propTypes = {
        className: PropTypes.string,
        createColumnDefinition: PropTypes.func.isRequired,
        data: PropTypes.array,
        defaultErrorMessage: PropTypes.string.isRequired,
        defaultNoDataText: PropTypes.string.isRequired,
        error: PropTypes.object,
        filter: tableFilterType,
        keyField: PropTypes.any.isRequired,
        layout: tableLayoutType.isRequired,
        loading: PropTypes.bool,
        loadingMessage: PropTypes.node,
        onFilterChange: PropTypes.func,
        onPageChange: PropTypes.func.isRequired,
        onRowClick: PropTypes.func,
        onRowDoubleClick: PropTypes.func,
        onSortChange: PropTypes.func.isRequired,
        paging: tablePagingType,
        register: PropTypes.func,
        selectRow: PropTypes.object.isRequired,
        sorting: tableSortingType.isRequired,
        tableHeaderClass: PropTypes.string,
        total: PropTypes.number.isRequired,
        totalClassName: PropTypes.string,
        transformNewStateOnChange: newStateTransformerType
    };

    static defaultProps = {
        data: [],
        loading: false
    };

    constructor(props) {
        super(props);
        this.tableWrapper = React.createRef();
        RU.invoke(['register'], [this.api], this.props);
    }

    componentDidMount() {
        this.adjustHeaderPadding();
    }

    componentDidUpdate() {
        this.adjustHeaderPadding();
    }

    componentWillUnmount() {
        document.removeEventListener('readystatechange', this.adjustHeaderPaddingListener);
    }

    handleOnTableChange = (type, newState) => {
        const {transformNewStateOnChange = {}} = this.props;
        const on = {
            filter: this.handleFilterChange,
            pagination: this.handlePageChange,
            sort: this.handleSortChange
        };
        const handler = on[type];
        if (handler) {
            const transformer = transformNewStateOnChange[type];
            if (transformer) {
                handler(transformer(newState));
            } else {
                handler(newState);
            }
        }
    };

    handleFilterChange = (newState) => {
        const {filter, onFilterChange} = this.props;
        const {filters} = newState;
        if (onFilterChange && !R.equals(filter, filters)) {
            const extractedFilter = extractFilters(filters);
            onFilterChange(mergeFilters(extractedFilter, filter));
        }
    };

    handlePageChange = (newState) => {
        const {onPageChange} = this.props;
        const {page, sizePerPage} = newState;
        onPageChange(page, sizePerPage);
    };

    handleSortChange = (newState) => {
        const {onSortChange, sorting: {name: oldSortField, order: oldSortOrder}} = this.props;
        const {sortField, sortOrder} = newState;
        if (!R.equals([oldSortField, oldSortOrder], [sortField, sortOrder])) {
            onSortChange(sortField, sortOrder);
        }
    };

    handleClickFilter = (event) => {
        event.stopPropagation();
    };

    adjustHeaderPadding = () => {
        if (document.readyState !== 'loading') {
            setHeaderPadding(this.tableWrapper);
        } else {
            document.addEventListener('readystatechange', this.adjustHeaderPaddingListener);
        }
    };

    adjustHeaderPaddingListener = () => {
        if (document.readyState !== 'loading') {
            this.adjustHeaderPadding();
        }
    };

    deselectAll = () => {
        this.tableWrapper.current.selectionContext.selected = [];
    };

    api = {
        adjustHeaders: this.adjustHeaderPadding,
        deselectAll: this.deselectAll,
        getSelectedRows: () => R.pathOr([], ['current', 'selectionContext', 'props', ' data'], this.tableWrapper),
        resetFilters: () => {
            R.pipe(
                R.keys,
                R.forEach((dataField) => {
                    this.filterRefsTable[dataField].current.applyNewModel('');
                })
            )(this.filterRefsTable);
        }
    };

    defaultSorted = this.props.sorting ?
        [
            {
                dataField: this.props.sorting.name,
                order: this.props.sorting.order
            }
        ] :
        null;

    filterRefsTable = {};

    rowEvents = {
        onClick: (evt, rowData) => RU.invoke(['onRowClick'], [rowData], this.props),
        onDoubleClick: (evt, rowData) => RU.invoke(['onRowDoubleClick'], [rowData], this.props)
    };

    buildSafeFilterConfig = (currentConfig) => {
        const {dataField: currentCol} = currentConfig;
        const {filter} = this.props;
        const value = R.prop(currentCol, filter);

        if (R.isNil(value)) {
            return currentConfig;
        }

        this.filterRefsTable[currentCol] = React.createRef();

        return {
            ...currentConfig,
            filter: customFilter(),
            filterRenderer: (onFilter) =>
                <div onClick={this.handleClickFilter}>
                    <XlReactWidgetFilter
                        metadata={{
                            placeholder:
                                currentConfig.filterPlaceholder || localeMessages.tableFilterInputPlaceholderFallback
                        }}
                        model={value}
                        onUpdate={onFilter}
                        ref={this.filterRefsTable[currentConfig.dataField]}
                    />
                </div>

        };
    };

    buildNoDataMessage = () => {
        const {defaultErrorMessage, defaultNoDataText} = this.props;
        return this.props.error ? defaultErrorMessage : defaultNoDataText;
    };

    createPaginationConfig = () => {
        if (!this.shouldShowPagination()) {
            return null;
        }

        const {
            total,
            paging: {
                page,
                nextPageText,
                pageStartIndex,
                resultsPerPage,
                pageButtonRenderer,
                pageSizes,
                paginationSize,
                prePageText,
                sizePerPageRenderer
            }
        } = this.props;

        return paginationFactory({
            hidePageListOnlyOnePage: true,
            nextPageText,
            page,
            pageButtonRenderer,
            pageStartIndex,
            paginationSize,
            paginationTotalRenderer: this.paginationTotalsRenderer,
            prePageText,
            showTotal: true,
            sizePerPage: resultsPerPage,
            sizePerPageList: pageSizes,
            sizePerPageRenderer: sizePerPageRenderer ? sizePerPageRenderer : this.sizePerPageRenderer,
            totalSize: total
        });
    };

    paginationTotalsRenderer = (start, to, total) => {
        const {totalClassName} = this.props;
        const text = template(localeMessages.reportPaginationTotals)({total});
        return totalClassName ? <div className={totalClassName}>{text}</div> : text;
    };

    sizePerPageRenderer = ({options, currSizePerPage, onSizePerPageChange}) =>
        <ButtonToolbar className="react-bs-table-sizePerPage-dropdown">
            <DropdownButton dropup={true} id="pagination-size-per-page" title={currSizePerPage}>
                {options.map((option) =>
                    <MenuItem key={option.text} onClick={() => onSizePerPageChange(option.page)}>
                        {option.text}
                    </MenuItem>
                )}
            </DropdownButton>
        </ButtonToolbar>
    ;

    shouldShowPagination = () => {
        const {paging} = this.props;
        return RU.isDefined(paging);
    };

    hidePagination = () => {
        const {paging, total} = this.props;
        if (RU.isDefined(paging)) {
            return total - paging.resultsPerPage < 1;
        }

        return true;
    };

    renderOverlay = () => {
        const {data, loading, loadingMessage} = this.props;
        return <XlReactWidgetSpinner message={R.isEmpty(data) ? loadingMessage : null} show={loading}/>;
    };

    render() {
        const {
            createColumnDefinition,
            layout,
            data,
            error,
            keyField,
            selectRow,
            loading,
            tableHeaderClass,
            className
        } = this.props;

        const hidePagination = {'hide-pagination': this.hidePagination()};
        const componentClassName = classNames('xl-react-widget-table', className, {
            hidePagination,
            loading
        });
        const tableHeaderClassName = classNames(tableHeaderClass);
        const tableClassName = classNames({error: RU.isDefined(error)});

        const columnDefinitionFactory = R.ifElse(
            shouldShowFilters,
            R.always(
                R.pipe(
                    createColumnDefinition,
                    this.buildSafeFilterConfig
                )
            ),
            R.always(createColumnDefinition)
        )(this.props);

        return (
            <div className={componentClassName}>
                <BootstrapTable
                    bordered={false}
                    classes={tableClassName}
                    columns={layout.map(columnDefinitionFactory)}
                    data={data}
                    defaultSorted={this.defaultSorted}
                    filter={filterFactory()}
                    headerClasses={tableHeaderClassName}
                    keyField={keyField}
                    noDataIndication={this.buildNoDataMessage}
                    onTableChange={this.handleOnTableChange}
                    pagination={this.createPaginationConfig()}
                    ref={this.tableWrapper}
                    remote={{
                        cellEdit: false,
                        filter: true,
                        pagination: RU.isDefined(this.props.paging),
                        sort: RU.isDefined(this.props.sorting)
                    }}
                    rowEvents={this.rowEvents}
                    selectRow={selectRow}
                    striped={true}
                    wrapperClasses="dropup"
                />
                {this.renderOverlay()}
            </div>
        );
    }
}

/* eslint-enable max-lines */
