import R from 'ramda';
import $ from 'jquery';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import InfiniteTree from 'xl-react-infinite-tree';
import RU from '../../utils/ramda-utils';
import 'xl-react-infinite-tree/dist/react-infinite-tree.css';
import './xl-react-widget-tree.less';
import {batchSelectNodes, multiSelectNodes} from './xl-react-widget-tree-service';

export class XlReactWidgetTree extends PureComponent {
    static propTypes = {
        autoOpen: PropTypes.bool,
        blocksInCluster: PropTypes.number,
        className: PropTypes.string,
        dndHandlers: PropTypes.shape({
            onDrag: PropTypes.func,
            onDragEnd: PropTypes.func,
            onDragEnter: PropTypes.func,
            onDragExit: PropTypes.func,
            onDragLeave: PropTypes.func,
            onDragOver: PropTypes.func,
            onDragStart: PropTypes.func,
            onDrop: PropTypes.func
        }),
        dontSelectClasses: PropTypes.arrayOf(PropTypes.string),
        droppableOptions: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.shape({
                accept: PropTypes.func.isRequired,
                drop: PropTypes.func.isRequired,
                hoverClass: PropTypes.string.isRequired
            })
        ]),
        loadNodes: PropTypes.func,
        model: PropTypes.array,
        noDataText: PropTypes.string,
        nodeHandlers: PropTypes.shape({
            onCloseNode: PropTypes.func,
            onOpenNode: PropTypes.func
        }),
        onClick: PropTypes.func,
        onDoubleClick: PropTypes.func,
        onNodesSelected: PropTypes.func,
        rowRenderer: PropTypes.func.isRequired,
        selectable: PropTypes.bool,
        selectedNodes: PropTypes.arrayOf(PropTypes.string),
        selectionMode: PropTypes.oneOf(['single', 'multi']),
        treeRef: PropTypes.func
    };

    static defaultProps = {
        blocksInCluster: 4,
        dontSelectClasses: [],
        selectable: true,
        selectedNodes: [],
        selectionMode: 'single'
    };

    UNSAFE_componentWillReceiveProps(nextProps) {
        const setSelectedState = R.curry((selected, nodeId) => {
            const node = this.tree.getNodeById(nodeId);
            if (node) {
                node.state.selected = selected;
                this.tree.updateNode(node, {}, {shallowRendering: true});
            }
        });

        const currentlySelectedNodes = R.pipe(
            R.filter(R.pathEq(['state', 'selected'], true)),
            R.pluck('id')
        )(R.values(this.tree.nodeTable.data));

        if (nextProps.selectedNodes && !R.equals(nextProps.selectedNodes, currentlySelectedNodes)) {
            R.pipe(R.difference, R.forEach(setSelectedState(false)))(currentlySelectedNodes, nextProps.selectedNodes);
            R.pipe(R.difference, R.forEach(setSelectedState(true)))(nextProps.selectedNodes, currentlySelectedNodes);
        }
    }

    handleMultiSelect = (currentNode) => {
        const result = multiSelectNodes(currentNode.id, this.props.selectedNodes);
        this.setBatchStartNode(currentNode);
        RU.invoke('onNodesSelected', [result], this.props);
    };

    handleBatchSelect = (toNode) => {
        const selected = batchSelectNodes(this.batchStartNodeId, toNode.id, this.props.selectedNodes, this.tree.nodes);
        this.setBatchStartNode(toNode);
        RU.invoke('onNodesSelected', [selected], this.props);
    };

    handleSingleSelect = (currentNode) => {
        this.setBatchStartNode(currentNode);
        RU.invoke('onNodesSelected', [[currentNode.id]], this.props);
    };

    handleNodeClick = (event) => {
        const currentNode = this.tree.getNodeById($(event.target).closest('.infinite-tree-item').data('id'));
        const dontSelect = (elem) =>
            R.any(R.includes(R.__, elem.className), [this.tree.options.togglerClass, ...this.props.dontSelectClasses]);

        if (!currentNode || dontSelect(event.target)) {
            return;
        }

        R.cond([
            [this.isSelectingMultiple, R.partial(this.handleMultiSelect, [currentNode])],
            [this.isSelectingBatch, R.partial(this.handleBatchSelect, [currentNode])],
            [R.T, R.partial(this.handleSingleSelect, [currentNode])]
        ])(event);
    };

    isMultiSelectMode = () => R.equals(this.props.selectionMode, 'multi');

    isSelectingMultiple = R.both(
        this.isMultiSelectMode,
        R.either(R.propEq('ctrlKey', true), R.propEq('metaKey', true))
    );

    isSelectingBatch = R.both(this.isMultiSelectMode, R.propEq('shiftKey', true));

    setBatchStartNode = (batchStartNode) => {
        this.batchStartNodeId = batchStartNode.id;
    };

    attachSelectionHandler = () => {
        if (R.equals(this.props.selectable, true)) {
            $(this.tree.contentElement).on('click', this.handleNodeClick);
        }
    };

    attachDndHandlers = () => {
        const {dndHandlers} = this.props;
        if (dndHandlers) {
            $(this.tree.contentElement).on('drag', dndHandlers.onDrag);
            $(this.tree.contentElement).on('dragend', dndHandlers.onDragEnd);
            $(this.tree.contentElement).on('dragenter', dndHandlers.onDragEnter);
            $(this.tree.contentElement).on('dragexit', dndHandlers.onDragExit);
            $(this.tree.contentElement).on('dragleave', dndHandlers.onDragLeave);
            $(this.tree.contentElement).on('dragover', dndHandlers.onDragOver);
            $(this.tree.contentElement).on('dragstart', dndHandlers.onDragStart);
            $(this.tree.contentElement).on('drop', dndHandlers.onDrop);
        }
    };

    nodeEventHandlerPairs = [
        ['openNode', 'onOpenNode'],
        ['closeNode', 'onCloseNode']
    ];

    attachNodeHandlers = () => {
        const {nodeHandlers} = this.props;
        if (nodeHandlers) {
            this.nodeEventHandlerPairs.forEach(([event, handlerName]) => {
                if (nodeHandlers[handlerName]) {
                    this.tree.on(event, nodeHandlers[handlerName]);
                }
            });
        }
    };

    assignTree = (tree) => {
        if (!tree) {
            return;
        }
        this.tree = tree.tree;
        if (this.props.treeRef) {
            this.props.treeRef(tree);
        }
        this.attachSelectionHandler();
        this.attachDndHandlers();
        this.attachNodeHandlers();
    };

    render() {
        const {
            autoOpen, blocksInCluster, className, droppableOptions,
            loadNodes, model, noDataText, onClick, onDoubleClick,
            rowRenderer
        } = this.props;

        return (
            <InfiniteTree
                autoOpen={autoOpen}
                blocksInCluster={blocksInCluster}
                className={className}
                data={model}
                droppable={droppableOptions}
                loadNodes={loadNodes}
                noDataText={noDataText}
                onClick={onClick}
                onDoubleClick={onDoubleClick}
                ref={this.assignTree}
                rowRenderer={rowRenderer}
                selectable={false}
            />
        );
    }
}
