import R from 'ramda';
import {
    prepareViewModel,
    addItem,
    filterItems,
    moveItem,
    sortItems,
    invalidCiValidator
} from '~/app/xl-react-dip/xl-react-dip-collection-of-ci/xl-react-dip-collection-of-ci-service';
import {localeMessages} from '~/app/services/locale-service';
import {WARNING} from '~/app/utils/ci-utils';

describe('Xl Dip Collection of CI Service', () => {
    describe('invalidCiValidator', () => {
        it('should return no errors if all the model items are valid', () => {
            const state = {validatedItems: [{valid: true}, {valid: true}, {valid: true}]};
            expect(invalidCiValidator(state)()).toEqual({});
        });

        it('should return warning if one of the cis is invalid', () => {
            const state = {validatedItems: [{valid: true}, {valid: false}, {valid: true}]};
            expect(invalidCiValidator(state)()).toEqual({
                condition: true,
                message: {
                    show: false,
                    text: localeMessages.invalidCiInTheListMessage,
                    type: WARNING
                }
            });
        });
    });

    describe('prepareViewModel', () => {
        const items = [
            {id: 1, label: 'First item'},
            {id: 2, label: 'Second Item'},
            {id: 3, label: 'Third'}
        ];

        it('should properly prepare view model', () => {
            expect(prepareViewModel(items, 'label')).toEqual([{
                model: ['First item', 0],
                valid: true
            }, {
                model: ['Second Item', 1],
                valid: true
            }, {
                model: ['Third', 2],
                valid: true
            }]);

            expect(prepareViewModel(['Item 1', 'Item 2', 'Item 3'])).toEqual([{
                model: ['Item 1', 0],
                valid: true
            }, {
                model: ['Item 2', 1],
                valid: true
            }, {
                model: ['Item 3', 2],
                valid: true
            }]);
        });
    });

    describe('filterItems', () => {
        const stringModel = [{
            model: ['Item 1', 0],
            valid: true
        }, {
            model: ['Item 2', 1],
            valid: true
        }, {
            model: ['Item 3', 2],
            valid: true
        }];

        const objModel = [{
            model: [{id: 'Item 1'}, 0],
            valid: true
        }, {
            model: [{id: 'Item 2'}, 1],
            valid: true
        }, {
            model: [{id: 'Item 3'}, 2],
            valid: true
        }];

        it('should filter items case insensitively', () => {
            expect(filterItems(stringModel)).toEqual(stringModel);
            expect(filterItems(stringModel, '1')).toEqual([{
                model: ['Item 1', 0],
                valid: true
            }]);
            expect(filterItems(objModel, '1', 'id')).toEqual([{
                model: [{id: 'Item 1'}, 0],
                valid: true
            }]);
        });
    });

    describe('addItem', () => {
        const input = [{id: 1, label: 'A'}];
        it('should add items (duplicates allowed when unique: false)', () => {
            expect(addItem(input, {id: 2, label: 'B'}, false)).toEqual([{id: 1, label: 'A'}, {id: 2, label: 'B'}]);
            expect(addItem(input, {id: 1, label: 'A'}, false)).toEqual([{id: 1, label: 'A'}, {id: 1, label: 'A'}]);
        });

        it('should add items (duplicates are NOT allowed when unique: true)', () => {
            expect(addItem(input, {id: 2, label: 'B'}, true)).toEqual([{id: 1, label: 'A'}, {id: 2, label: 'B'}]);
            expect(addItem(input, {id: 1, label: 'A'}, true)).toEqual([{id: 1, label: 'A'}]);
        });
    });

    describe('moveItem', () => {
        it('should move item within an array', () => {
            const input = [
                {id: 1, label: 'A'},
                {id: 2, label: 'B'},
                {id: 3, label: 'C'}
            ];
            expect(moveItem(2, 0, input)).toEqual([
                {id: 3, label: 'C'},
                {id: 1, label: 'A'},
                {id: 2, label: 'B'}
            ]);
            expect(moveItem(1, 2, input)).toEqual([
                {id: 1, label: 'A'},
                {id: 3, label: 'C'},
                {id: 2, label: 'B'}
            ]);
        });
    });

    describe('sortItems', () => {
        it('should not fail on undefined or empty input', () => {
            expect(sortItems([])).toEqual([]);
            expect(sortItems(undefined)).toEqual(undefined);
        });
        it('should sort CI`s alphabetically with ignoring the name of root node', () => {
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];
            const expected = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            expect(sortItems(input)).toEqual(expected);
        });
        it('should sort objects alphabetically with ignoring the name of root node', () => {
            const transformToObject = R.map((item) => ({id: item, type: 'someType'}));
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];

            const sortedOutput = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            const expected = transformToObject(sortedOutput);
            expect(sortItems(transformToObject(input))).toEqual(expected);
        });
        it('should sort objects (with custom objectLabel) alphabetically with ignoring the name of root node', () => {
            const transformToObject = R.map((item) => ({name: item, type: 'someType'}));
            const input = ['Environments/TestApp/1.0', 'Applications/TestApp/2.0', 'Configuration/TestApp/1.2'];

            const sortedOutput = ['Environments/TestApp/1.0', 'Configuration/TestApp/1.2', 'Applications/TestApp/2.0'];
            const expected = transformToObject(sortedOutput);
            expect(sortItems(transformToObject(input), 'name')).toEqual(expected);
        });
    });
});
