import React from 'react';
import {mount} from 'enzyme';

import Helper from '../../validations-spec-helper';
import {XlReactDipString} from '~/app/xl-react-dip/xl-react-dip-string/xl-react-dip-string';
import {ERROR} from '~/app/utils/ci-utils';

describe('XlReactDipString-default', () => {
    const defaultProps = {
        metadata: {
            description: 'Its a string',
            label: 'some label',
            placeholder: 'some placeholder'
        }
    };

    it('should have all elements present', () => {
        const wrapper = mount(<XlReactDipString {...defaultProps}/>);
        expect(wrapper.find('XlReactWidgetString XlReactWidgetLabel')).toExist();
        expect(wrapper.find('.xl-react-components-input input')).toExist();
        expect(wrapper.find('.xl-react-components-label label')).toExist();
        expect(wrapper.find('.xl-react-components-description')).toExist();
    });

    it('should bind entered value', () => {
        const wrapper = mount(<XlReactDipString {...defaultProps} model="test value"/>);
        expect(wrapper.prop('model')).toBe('test value');
    });

    it('should erase entered value', () => {
        const onModelChange = jest.fn();
        const wrapper = mount(<XlReactDipString {...defaultProps} model="test text" onModelChange={onModelChange}/>);
        expect(wrapper.prop('model')).toBe('test text');
        expect(wrapper.find('.xl-react-components-input input')).toHaveValue('test text');
        expect(wrapper.find('.close-icon')).toExist();
        wrapper.find('.close-icon').simulate('click');
        setTimeout(() => {
            expect(wrapper.find('.close-icon')).not.toExist();
            expect(onModelChange).toBeCalled();
            expect(onModelChange).toBeCalledWith('', false);
        }, 500);
    });

    it('should display all modes correctly', () => {
        const wrapper = mount(<XlReactDipString {...defaultProps} model="te"/>);
        wrapper.setProps({displayMode: ''});
        const input = () => wrapper.find('.xl-react-components-input input');
        expect(input()).not.toBeDisabled();

        wrapper.setProps({displayMode: 'edit'});
        expect(input()).not.toBeDisabled();

        wrapper.setProps({displayMode: 'visual'});
        expect(input().props().readOnly).toEqual(true);

        wrapper.setProps({
            disabled: true,
            displayMode: 'edit'
        });
        expect(input().props().readOnly).toEqual(true);
    });

    it('should honour custom validations', () => {
        const validator = (model) => ({
            condition: !model.toLowerCase().startsWith('depl-'),
            message: {show: false, text: 'Your ticket number should start with DEPL-', type: ERROR}
        });
        const {changeInput, errMsg} = new Helper('.xl-react-widget-string').buildFrom(<XlReactDipString
            {...defaultProps}
            validators={[validator]}
        />);
        changeInput('REL-1234');
        setTimeout(() => {
            expect(errMsg()).toHaveText('Your ticket number should start with DEPL-');

            changeInput('DEPL-1234');
            expect(errMsg()).not.toExist();
        }, 500);
    });
});

describe('XlReactDipString-with-default-value', () => {
    const defaultProps = {
        metadata: {
            default: 'default-value',
            description: 'Its a string',
            label: 'some label',
            placeholder: 'some placeholder'
        }
    };

    it('should have default value binded', () => {
        const onModelChange = jest.fn();
        const wrapper = mount(<XlReactDipString {...defaultProps} model="" onModelChange={onModelChange}/>);
        setTimeout(() => {
            expect(onModelChange).toBeCalledWith('default-value', true);
            expect(wrapper.find('.xl-react-components-input input')).toHaveValue('default-value');
        }, 500);
    });

    it('should bind entered value', () => {
        const wrapper = mount(<XlReactDipString {...defaultProps} model="test value"/>);
        expect(wrapper.prop('model')).toBe('test value');
    });
});

describe('XlReactDipString-with-default-error-message', () => {
    it('should always show error received in props', () => {
        const onValidityChangeFn = jest.fn();
        const onModelChangeFn = jest.fn();
        const wrapper = mount(<XlReactDipString
            message={{show: false, text: 'this is an error', type: ERROR}}
            metadata={{required: true}}
            onModelChange={onModelChangeFn}
            onValidityChange={onValidityChangeFn}
        />);

        const input = wrapper.find('.xl-react-components-input-full input');
        expect(input).toExist();
        input.simulate('change', {target: {value: 'value'}});
        input.simulate('change', {target: {value: ''}});
        setTimeout(() => {
            expect(wrapper.find('.xl-react-components-input-full input')).toHaveClassName('invalid');
            const message = wrapper.find('.xl-react-components-input-full .error');
            expect(message).toExist();
            expect(message).toHaveText('this is an error');
        }, 500);
    });
});
