import React from 'react';
import {mount} from 'enzyme';
import R from 'ramda';
import {XlReactTabSystemContainer} from '~/app/xl-react-tab-system/xl-react-tab-system-container';
import {XlReactWidgetString} from '~/app/xl-react-widget/xl-react-widget-string/xl-react-widget-string';
import {XlReactWidgetAutocomplete} from '~/app/xl-react-widget/xl-react-widget-autocomplete/xl-react-widget-autocomplete'; // eslint-disable-line max-len

const autocompleteHandlers = {
    addCandidates:
        () => Promise.resolve(R.map((id) => ({id}), ['Grape', 'Orange', 'Apple']))
};

describe('<XlReactTabSystemContainer />', () => {
    const buildContext = (data, activeTabId) => {
        const context = {
            activeTabId,
            data,
            handleChangeActiveTabId(newId) {
                this.activeTabId = newId;
                this.wrapper.setProps({activeTabId: this.activeTabId});
            },
            handleChangeData(newData) {
                this.data = newData;
                this.wrapper.setProps({data: this.data});
            },
            onTabRegister(api) {
                this.tabsApi = api;
            },
            registerWrapper(wrapper) {
                this.wrapper = wrapper;
            },
            tabsApi: undefined
        };
        context.onTabRegister = context.onTabRegister.bind(context);
        context.handleChangeData = context.handleChangeData.bind(context);
        context.handleChangeActiveTabId = context.handleChangeActiveTabId.bind(context);
        context.registerWrapper = context.registerWrapper.bind(context);
        return context;
    };

    it('should render the tab system with one tab', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];
        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-string': {
                onClose: R.identity,
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };
        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const tabSystem = wrapper.find('.xl-react-tab-system');
        expect(tabSystem).toBeDefined();
        expect(tabSystem.find('NavItem')).toBeDefined();
        expect(tabSystem.find('.header-column .btn-group-backdrop .btn-add-new-tab')).toHaveLength(1);
    });

    it('should render the tab system with multiple tabs', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }, {
            pristine: true,
            tabId: '2',
            title: 'xl-react-widget-select',
            viewType: 'xl-react-widget-select'
        }];
        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-select': {
                onClose: R.identity,
                renderContent: () => <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    metadata={{required: true}}
                />
            },
            'xl-react-widget-string': {
                onClose: R.identity,
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const tabSystem = wrapper.find('.xl-react-tab-system');
        expect(tabSystem).toBeDefined();
        expect(tabSystem.find('NavItem')).toHaveLength(2);
        expect(tabSystem.find('.header-column .btn-group-backdrop .btn-add-new-tab')).toHaveLength(1);
    });

    it('should close the currently open tab', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }, {
            pristine: true,
            tabId: '2',
            title: 'xl-react-widget-select',
            viewType: 'xl-react-widget-select'
        }];
        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-select': {
                onClose: () => context.tabsApi.removeTab(context.tabsApi),
                renderContent: () => <XlReactWidgetAutocomplete
                    handlers={autocompleteHandlers}
                    metadata={{required: true}}
                />
            },
            'xl-react-widget-string': {
                onClose: (tabId) => context.tabsApi.removeTab(tabId),
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);

        context.registerWrapper(wrapper);
        const tabSystem = wrapper.find('.xl-react-tab-system');
        const tabs = tabSystem.find('.nav.nav-tabs');
        expect(tabSystem.find('NavItem')).toHaveLength(2);
        expect(tabSystem.find('.header-column .btn-group-backdrop .btn-add-new-tab')).toHaveLength(1);

        const closeButton = tabs.find('.close-icon');
        expect(closeButton).toHaveLength(2);
        closeButton.at(0).simulate('click');
        expect(tabSystem.find('.nav.nav-tabs')).toHaveLength(1);
    });

    it('should open a new tab after add tab is called', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];
        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-string': {
                onClose: (tabId) => context.tabsApi.removeTab(tabId),
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const newTabId = context.tabsApi.addTab({
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        });

        const tabSystem = wrapper.find('.xl-react-tab-system');

        expect(tabSystem.find('NavItem')).toHaveLength(2);
        expect(tabSystem.find('.header-column .btn-group-backdrop .btn-add-new-tab')).toHaveLength(1);

        const {tabId} = context.tabsApi.getActiveTab();
        expect(tabId).toBe(newTabId);
    });

    it('should change title of an existing tab', () => {
        const initialData = [{
            fullTitle: 'xl-react-widget-string',
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];

        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-string': {
                onClose: (tabId) => context.tabsApi.removeTab(tabId),
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const tabSystem = wrapper.find('.xl-react-tab-system');
        expect(tabSystem).toBeDefined();
        expect(tabSystem.find('.tab-title').prop('title')).toBe('xl-react-widget-string');

        context.tabsApi.changeTitle('1', 'full new title', 'new title');
        const updatedTab = context.tabsApi.getTab('1');
        expect(updatedTab.title).toBe('new title');
    });

    it('should change loading state of tab', () => {
        const data = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];

        const tabDefinitions = {
            'xl-react-widget-string': {
                renderContent: () =>
                    <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const context = buildContext(data);
        const wrapper = mount(<XlReactTabSystemContainer
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);
        const tabSystem = () => wrapper.find('.xl-react-tab-system');
        expect(tabSystem()).toBeDefined();

        context.tabsApi.changeLoading('1', true);
        expect(tabSystem().find('XlReactWidgetSpinner').length).toBe(1);

        context.tabsApi.changeLoading('1', false);
        expect(tabSystem().find('XlReactWidgetSpinner').length).toBe(0);
    });

    it('should change icon of the component', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];

        const context = buildContext(initialData, '1');

        const tabDefinitions = {
            'xl-react-widget-string': {
                onClose: (tabId) => context.tabsApi.removeTab(tabId),
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const tabSystem = wrapper.find('.xl-react-tab-system');
        expect(tabSystem).toBeDefined();
        const {tabId} = context.tabsApi.getActiveTab();
        context.tabsApi.changeIconUrl(tabId, 'icon-url');
        const updatedTab = context.tabsApi.getTab(tabId);
        expect(updatedTab.iconUrl).toBe('icon-url');
    });

    it('should render selected tab renderContent in the pane', () => {
        const initialData = [{
            pristine: true,
            tabId: '1',
            title: 'xl-react-widget-string',
            viewType: 'xl-react-widget-string'
        }];
        const context = buildContext(initialData);

        const tabDefinitions = {
            'xl-react-widget-string': {
                onClose: (tabId) => context.tabsApi.removeTab(tabId),
                renderContent: () => <XlReactWidgetString metadata={{required: true}} model="apple"/>
            }
        };

        const wrapper = mount(<XlReactTabSystemContainer
            activeTabId={context.activeTabId}
            data={context.data}
            onChangeActiveTabId={context.handleChangeActiveTabId}
            onChangeData={context.handleChangeData}
            register={context.onTabRegister}
            tabDefinitions={tabDefinitions}
        />);
        context.registerWrapper(wrapper);

        const tabSystem = wrapper.find('.xl-react-tab-system');
        expect(tabSystem.find('.xl-react-components-input-full').find('input').prop('value')).toBe('apple');
    });
});
