import React from 'react';
import {mount, ReactWrapper} from 'enzyme';
import * as KeyCodes from 'keycode-js';

import {Modal} from 'react-bootstrap';
import ReactTestUtils from 'react-dom/test-utils';
import {XlReactWidgetConfirmationDialog} from '~/app/xl-react-widget/xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog'; // eslint-disable-line max-len
import {confirmationDialogSkins} from '~/app/xl-react-widget/xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog-skins'; // eslint-disable-line max-len

describe('XlReactWidgetConfirmationDialog', () => {
    const defaultButtons = [
        {action: 'confirm', className: 'xl-primary', keyCodes: [KeyCodes.KEY_RETURN], label: 'Yes'},
        {action: 'reject', className: 'xl-secondary', keyCodes: [KeyCodes.KEY_ESCAPE], label: 'No'}
    ];

    const disabledButton = [
        {action: 'confirm', className: 'xl-primary', disabled: true, keyCodes: [KeyCodes.KEY_RETURN], label: 'Yes'}
    ];

    it('by default modal dialog shouldn\'t be visible', () => {
        const wrapper = mount(<XlReactWidgetConfirmationDialog show={false}/>);
        expect(wrapper.find(Modal).exists()).toBe(false);
    });

    it('should be possible to make it visible', () => {
        const wrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={defaultButtons}
            show={true}
        />);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const modal = new ReactWrapper(dialog, dialog);
        expect(modal.find('.modal-dialog')).toBeDefined();
        expect(modal.find('.modal-content')).toBeDefined();
        expect(modal.find('.confirmation-content')).toBeDefined();
        expect(modal.find('.modal-footer')).toBeDefined();
        expect(modal.find('.dialog-buttons')).toBeDefined();

        expect(modal.find('button.xl-primary')).toBeDefined();
        expect(modal.find('button.xl-secondary')).toBeDefined();
    });

    it('should be able to set button labels and propagate events of those buttons', () => {
        const handleCloseFn = jest.fn();

        const wrapper = mount(
            <XlReactWidgetConfirmationDialog
                buttons={defaultButtons}
                onButtonClick={handleCloseFn}
                show={true}>
                <span className="my-element"/>
            </XlReactWidgetConfirmationDialog>
        );
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const modal = new ReactWrapper(dialog, dialog);
        expect(modal.find('.my-element')).toBeDefined();

        const confirmButton = dialog.getElementsByClassName('xl-primary')[0];
        const rejectButton = dialog.getElementsByClassName('xl-secondary')[0];

        expect(confirmButton.textContent).toBe('Yes');
        expect(rejectButton.textContent).toBe('No');

        ReactTestUtils.Simulate.click(rejectButton);
        expect(handleCloseFn).toHaveBeenCalledTimes(1);
        expect(handleCloseFn.mock.calls).toEqual([['reject']]);

        ReactTestUtils.Simulate.click(confirmButton);
        expect(handleCloseFn).toHaveBeenCalledTimes(2);
        expect(handleCloseFn.mock.calls).toEqual([['reject'], ['confirm']]);
    });

    it('should render child component inside modal dialog', () => {
        const wrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={defaultButtons}
            show={true}>
            <span className="my-element"/></XlReactWidgetConfirmationDialog>);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const modal = new ReactWrapper(dialog, dialog);
        expect(modal.find('.my-element')).toBeDefined();
    });

    it('should be possible to perform primary action on keyboard ENTER press', () => {
        const handleCloseFn = jest.fn();
        const wrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={defaultButtons}
            onButtonClick={handleCloseFn}
            show={true}
        />);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        ReactTestUtils.Simulate.keyDown(dialog, {key: 'Enter', keyCode: 13, which: 13});
        expect(handleCloseFn.mock.calls).toEqual([['confirm']]);
        expect(handleCloseFn).toHaveBeenCalledTimes(1);
    });

    it('should be possible to perform secondary action on keyboard ESC press', () => {
        const handleCloseFn = jest.fn();
        const wrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={defaultButtons}
            onButtonClick={handleCloseFn}
            show={true}
        />);
        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        ReactTestUtils.Simulate.keyDown(dialog, {key: 'Esc', keyCode: 27, which: 27});
        expect(handleCloseFn.mock.calls).toEqual([['reject']]);
        expect(handleCloseFn).toHaveBeenCalledTimes(1);
    });

    it('button should be disabled and keyboard actions shouldn\'t respond', () => {
        const handleButtonFn = jest.fn();
        const wrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={disabledButton}
            onButtonClick={handleButtonFn}
            show={true}
        />);

        const dialog = wrapper.find(Modal).instance()._modal.getDialogElement();
        const button = dialog.getElementsByClassName('xl-primary')[0];
        expect(button.disabled).toBe(true);

        ReactTestUtils.Simulate.keyDown(dialog, {key: 'Enter', keyCode: 13, which: 13});
        expect(handleButtonFn).toHaveBeenCalledTimes(0);
    });

    it('should be render accordingly to its skin', () => {
        const handleButtonFn = jest.fn();
        const handleCheckboxFn = jest.fn();
        const checkboxWrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={disabledButton}
            onButtonClick={handleButtonFn}
            onCheckboxClick={handleCheckboxFn}
            show={true}
            skin={confirmationDialogSkins.CHECKBOX}
        />);

        const dialog = checkboxWrapper.find(Modal).instance()._modal.getDialogElement();
        const checkboxLabel = checkboxWrapper.find('label');
        expect(checkboxLabel.text()).toBe('Create another');
        const checkbox = dialog.getElementsByTagName('input')[0];
        ReactTestUtils.Simulate.click(checkbox);
        expect(handleCheckboxFn).toHaveBeenCalledTimes(1);

        const defaultWrapper = mount(<XlReactWidgetConfirmationDialog
            buttons={disabledButton}
            onButtonClick={handleButtonFn}
            onCheckboxClick={handleCheckboxFn}
            show={true}
            skin={confirmationDialogSkins.DEFAULT}
        />);

        const defaultDialog = defaultWrapper.find(Modal).instance()._modal.getDialogElement();
        expect(defaultDialog.getElementsByTagName('input')[0]).toBeUndefined();
    });
});
