import R from 'ramda';
import React from 'react';
import {mount} from 'enzyme/build/index';
import {XlReactWidgetTable} from '~/app/xl-react-widget/xl-react-widget-table/xl-react-widget-table';

describe('XlReactWidgetTable', () => {
    const buildClassName = (dataKey) => `xl-react-table-${dataKey}`;

    const fieldFormatters = {
        completionDate: R.identity,
        startDate: R.identity,
        status: R.identity,
        type: R.identity
    };

    const createColumnDefinition = (headerDef) => {
        const dataFormat = R.propOr(R.identity, headerDef.dataKey, fieldFormatters);

        return {
            classes: buildClassName(headerDef.dataKey),
            dataField: headerDef.dataKey,
            formatter: dataFormat,
            headerClasses: buildClassName(headerDef.dataKey),
            headerTitle: false,
            sort: true,
            text: headerDef.title
        };
    };

    const selectRow = {
        className: 'row-selected',
        clickToSelect: true,
        hideSelectColumn: true,
        mode: 'radio'
    };

    const layout = [{
        dataKey: 'firstname',
        id: '1',
        sortKey: 'firstname',
        title: 'Firstname'
    }, {
        dataKey: 'address',
        id: '2',
        sortKey: 'address',
        title: 'Address'
    }, {
        dataKey: 'city',
        id: '3',
        sortKey: 'city',
        title: 'City'
    }, {
        dataKey: 'state',
        id: '4',
        sortKey: 'state',
        title: 'State'
    }];

    const data = [{
        address: '1236 Some Street',
        city: 'San Francisco',
        firstname: 'John A. Smith',
        state: 'CA'
    }, {
        address: '3271 Another Ave',
        city: 'New York',
        firstname: 'Joan B. Jones',
        state: 'NY'
    }, {
        address: '9996 Random Road',
        city: 'Los Angeles',
        firstname: 'Bob C. Uncle',
        state: 'CA'
    }, {
        address: '1623 Some Street',
        city: 'San Francisco',
        firstname: 'John D. Smith',
        state: 'CA'
    }, {
        address: '9889 Random Road',
        city: 'Los Angeles',
        firstname: 'Bob F. Uncle',
        state: 'CA'
    }];

    let componentData = data;

    const handlePageChange = (page, sizePerPage) => {
        const currentIndex = (page - 1) * sizePerPage;
        componentData = data.slice(currentIndex, currentIndex + sizePerPage);
    };

    const handleSortChange = (sortName, sortOrder) => {
        const sort = R.sortWith([
            sortOrder === 'asc' ? R.ascend(R.prop(sortName)) : R.descend(R.prop(sortName))
        ]);
        componentData = sort(data);
    };

    const handleRowDoubleClick = (row) => row;

    const handleRowClick = (row) => row;

    const paging = {
        nextPageText: 'Next',
        page: 1,
        pageSizes: [{
            text: '2',
            value: 2
        }],
        pageStartIndex: 1,
        paginationPosition: 'bottom',
        prePageText: 'Prev',
        resultsPerPage: 1
    };

    const sorting = {name: 'firstname', order: 'desc'};
    const total = data.length;
    const keyField = 'firstname';

    it('should render table header columns correctly', () => {
        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={handlePageChange}
            onRowDoubleClick={handleRowDoubleClick}
            onSortChange={handleSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);

        expect(wrapper.find('.xl-react-table-address')).toExist();
        expect(wrapper.find('.xl-react-table-firstname')).toExist();
        expect(wrapper.find('.xl-react-table-city')).toExist();
        expect(wrapper.find('.xl-react-table-state')).toExist();
    });

    it('should contain 5 pages and 1 item per page', () => {
        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={handlePageChange}
            onRowDoubleClick={handleRowDoubleClick}
            onSortChange={handleSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);
        const additionalPaginationItems = 1;
        const numberOfPages = 5 + additionalPaginationItems;

        expect(wrapper.find('.pagination').children().length).toBe(numberOfPages);
        wrapper.props().onPageChange(1, paging.resultsPerPage);
        expect(wrapper.find('tbody').length).toBe(1);
        wrapper.props().onPageChange(2, paging.resultsPerPage);
        expect(wrapper.find('tbody').length).toBe(1);
        wrapper.props().onPageChange(3, paging.resultsPerPage);
        expect(wrapper.find('tbody').length).toBe(1);
        wrapper.props().onPageChange(4, paging.resultsPerPage);
        expect(wrapper.find('tbody').length).toBe(1);
        wrapper.props().onPageChange(5, paging.resultsPerPage);
        expect(wrapper.find('tbody').length).toBe(1);
    });

    it('should call handleSortChange if onSortChange is invoked', () => {
        const mockSortChange = jest
            .fn()
            .mockImplementation(handleSortChange);

        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={handlePageChange}
            onRowDoubleClick={handleRowDoubleClick}
            onSortChange={mockSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);

        wrapper.props().onSortChange('firstname', 'desc');
        expect(mockSortChange).toBeCalledWith('firstname', 'desc');
    });

    it('should call handlePageChange if onPageChange is invoked', () => {
        const mockPageChange = jest
            .fn()
            .mockImplementation(handlePageChange);

        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={mockPageChange}
            onRowDoubleClick={handleRowDoubleClick}
            onSortChange={handleSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);

        wrapper.props().onPageChange(1, paging.resultsPerPage);
        expect(mockPageChange).toBeCalledWith(1, paging.resultsPerPage);
    });

    it('should call handleRowDoubleClick if onRowDoubleClick is invoked', () => {
        const mockRowDoubleClick = jest
            .fn()
            .mockImplementation(handleRowDoubleClick);

        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={handlePageChange}
            onRowDoubleClick={mockRowDoubleClick}
            onSortChange={handleSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);

        wrapper.props().onRowDoubleClick();
        expect(mockRowDoubleClick).toBeCalled();
    });

    it('should call handleRowClick if onRowClick is invoked', () => {
        const mockRowClick = jest
            .fn()
            .mockImplementation(handleRowClick);

        const wrapper = mount(<XlReactWidgetTable
            createColumnDefinition={createColumnDefinition}
            data={componentData}
            defaultErrorMessage={'Error occurred while fetching report'}
            defaultNoDataText={'No data available for the selected filter'}
            fieldFormatters={fieldFormatters}
            keyField={keyField}
            layout={layout}
            onPageChange={handlePageChange}
            onRowClick={mockRowClick}
            onSortChange={handleSortChange}
            paging={paging}
            selectRow={selectRow}
            sorting={sorting}
            total={total}
        />);

        wrapper.props().onRowClick();
        expect(mockRowClick).toBeCalled();
    });
});
