import {
    filterTree,
    flatTree,
    initTree,
    setSelectedNode,
    toggleNode
} from '~/app/xl-react-widget/xl-react-widget-tree-select/tree-utils';

describe('xl-react-widget-tree-select', () => {
    describe('tree-utils', () => {
        describe('initTree', () => {
            it('should init tree nodes with additional fields', () => {
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: ['CHILD_NODE'],
                        id: 'SHOULD_BE_EXTENDED'
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: ['CHILD_NODE'],
                        id: 'SHOULD_BE_EXTENDED',
                        isExpanded: false,
                        isParent: true
                    }
                ];
                expect(initTree(tree)).toEqual(expected);
            });
        });

        describe('setSelectedNode', () => {
            it('should set selected node', () => {
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{id: 'SELECTED_NODE'}],
                        id: 'SHOULD_BE_EXPANDED'
                    },
                    {
                        children: [{id: 'CHILD_NODE'}],
                        id: 'SHOULD_BE_COLLAPSED',
                        isExpanded: true
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME', isExpanded: false},
                    {
                        children: [{
                            id: 'SELECTED_NODE',
                            isExpanded: false,
                            isSelected: true
                        }],
                        id: 'SHOULD_BE_EXPANDED',
                        isExpanded: true
                    },
                    {
                        children: [{id: 'CHILD_NODE', isExpanded: false}],
                        id: 'SHOULD_BE_COLLAPSED',
                        isExpanded: false
                    }
                ];
                expect(setSelectedNode(tree, 'SELECTED_NODE')).toEqual(expected);
            });
        });

        describe('flatTree', () => {
            it('should flat tree', () => {
                const tree = [
                    {id: '1'},
                    {
                        children: [
                            {
                                children: [{id: '4'}],
                                id: '3',
                                isExpanded: true
                            }
                        ],
                        id: '2',
                        isExpanded: true
                    },
                    {
                        children: [{id: 'SHOULD_BE_IGNORED'}],
                        id: '5',
                        isExpanded: false
                    },
                    {id: '6'}
                ];
                const expected = [
                    {depth: 0, id: '1'},
                    {depth: 0, id: '2', isExpanded: true},
                    {depth: 1, id: '3', isExpanded: true},
                    {depth: 2, id: '4'},
                    {depth: 0, id: '5', isExpanded: false},
                    {depth: 0, id: '6'}
                ];
                expect(flatTree(tree)).toEqual(expected);
            });
        });

        describe('toggleNode', () => {
            it('should expand node', () => {
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_EXPANDED',
                            isExpanded: false
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_EXPANDED',
                            isExpanded: true
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                expect(toggleNode(tree, 'SHOULD_BE_EXPANDED')).toEqual(expected);
            });

            it('should collapse node', () => {
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_COLLAPSED',
                            isExpanded: true
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_COLLAPSED',
                            isExpanded: false
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                expect(toggleNode(tree, 'SHOULD_BE_COLLAPSED')).toEqual(expected);
            });

            it('should use force to expand node', () => {
                const FORCE = true;
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_EXPANDED',
                            isExpanded: true
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_EXPANDED',
                            isExpanded: FORCE
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                expect(toggleNode(tree, 'SHOULD_BE_EXPANDED', FORCE)).toEqual(expected);
            });

            it('should use force to collapse node', () => {
                const FORCE = false;
                const tree = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_COLLAPSED',
                            isExpanded: false
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                const expected = [
                    {id: 'SHOULD_REMAIN_THE_SAME'},
                    {
                        children: [{
                            id: 'SHOULD_BE_COLLAPSED',
                            isExpanded: FORCE
                        }],
                        id: 'SHOULD_REMAIN_THE_SAME'
                    }
                ];
                expect(toggleNode(tree, 'SHOULD_BE_COLLAPSED', FORCE)).toEqual(expected);
            });
        });

        describe('filterTree', () => {
            it('should filter tree nodes by criteria', () => {
                const tree = [
                    {title: 'SHOULD_BE_FILTRED'},
                    {
                        children: [{
                            children: [
                                {title: 'MATCH_FILTER_CRITERIA'},
                                {title: 'SHOULD_BE_FILTRED'}
                            ],
                            title: 'SHOULD_REMAIN'
                        }],
                        title: 'SHOULD_REMAIN'
                    },
                    {title: 'SHOULD_BE_FILTRED'}
                ];
                const expected = [
                    {
                        children: [{
                            children: [{title: 'MATCH_FILTER_CRITERIA'}],
                            isExpanded: true,
                            isParent: true,
                            title: 'SHOULD_REMAIN'
                        }],
                        isExpanded: true,
                        isParent: true,
                        title: 'SHOULD_REMAIN'
                    }
                ];
                expect(filterTree(tree, 'MATCH_FILTER_CRITERIA')).toEqual(expected);
            });
        });
    });
});
